package com.kexun.controller;


import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.kexun.common.utils.Result;
import com.kexun.entity.ReportFileEntity;
import com.kexun.service.ReportFileService;
import lombok.extern.java.Log;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.Base64;
import java.util.List;

@RestController
@RequestMapping("/admin/report")
@Log
public class ReportController {


    /**
     * 文件目录，解析后的文件目录
     */
    @Value("${report.file.xmlSourcePath}")
    private String xmlSourcePath;
    /**
     * 文件目录，解析后的文件目录
     */
    @Value("${report.file.xmlTargetPath}")
    private String xmlTargetPath;

    /**
     * 模型文件目标文件
     */
    @Value("${report.file.modelTargetPath}")
    private String modelPath;

    /**
     * 存放JSON文件路径
     */
    @Value("${report.file.jsonTargetPath}")
    private String jsonTargetPath;

    @Autowired
    private ReportFileService reportFileService;


    @GetMapping("reportFileList")
    public Result businessToDoList(@RequestParam String businessNum) {
        JSONObject resJson=new JSONObject();
        List<ReportFileEntity> list= reportFileService.QueryFileByNum(businessNum);
        resJson.put("report_file_list",list);

        return Result.success("ok",resJson);
    }


    /**
     * xml及json文件下载
     * @param filename
     * @return
     * @throws IOException
     */
    @GetMapping("/download/{filename}")
    public Result downloadFile(@PathVariable("filename") String filename) throws IOException {
        String downFilePath = xmlTargetPath + filename;
        if (filename.endsWith(".txt")){
            downFilePath = jsonTargetPath + filename;
        }
        File file = new File(downFilePath);
        if (file.exists()) {
            byte[] fileContent = Files.readAllBytes(file.toPath());
            String base64 = Base64.getEncoder().encodeToString(fileContent);
            JSONObject resJson = new JSONObject();
            resJson.put("file", base64);
            return Result.success("ok", resJson);
        } else {
            return Result.error("文件不存在!");
        }

    }

    /**
     * 模型文件下载
     * @param modname
     * @return
     * @throws IOException
     */
    @GetMapping("/downloadMod/{modname}")
    public Result downloadMod(@PathVariable("modname") String modname) throws IOException {
        String downFilePath = modelPath+ modname;

        File file = new File(downFilePath);
        if (file.exists()) {

            byte[] fileContent = Files.readAllBytes(file.toPath());
            String base64 = Base64.getEncoder().encodeToString(fileContent);
            JSONObject resJson = new JSONObject();
            resJson.put("file", base64);
            return Result.success("ok", resJson);
        } else {

            return Result.error("文件不存在!");
        }

    }

    @PostMapping("del")
    public Result del(@RequestBody String json) {
        JSONObject req = JSON.parseObject(json);
        String id = req.getString("id");
        String file_name = req.getString("file_name");
        String type = req.getString("type");

        ReportFileEntity reportFileEntity = reportFileService.getById(id);

        //判断文件是哪种
        System.out.println("file_name is "+ file_name);
        if(type.equals("1")) {
            if (file_name.endsWith("xml")) {
                reportFileEntity.setFileNameXml("");
            } else if (file_name.endsWith("txt")) {
                reportFileEntity.setFileNameTxt("");
            }
        }
        else {
            reportFileEntity.setFileNameModel("");
        }
        File file = new File(xmlTargetPath +file_name);
        file.delete();
        reportFileService.saveOrUpdate(reportFileEntity);
        return Result.success();
    }


    /**
     * 文件上传
     * 原来的逻辑是新增，2024.1.2之后改为修改 如果没有文件，改为新增，方法需要重写
     * 目前仍需改：若已有文件，需将原有数据删除
     * @param json
     * @return
     */
    @PostMapping("fileReportAdd")
    public Result fileReportAdd(@RequestBody String json)
    {
        JSONObject req = JSON.parseObject(json);
        long headInfId=Long.valueOf(req.getString("headInfId")) ;
        long businessId=Long.valueOf(req.getString("businessId")) ;
        String s = req.getString("fileNameTxt");
        ReportFileEntity reportFileEntity = reportFileService.findByHeadInfId(headInfId);
        if(reportFileEntity==null) {
            reportFileEntity = new ReportFileEntity();
            reportFileEntity.setHeadInfId(headInfId);
            reportFileEntity.setRptNo(req.getString("rptNo"));
            reportFileEntity.setBusinessId(businessId);
            reportFileEntity.setBusinessNum(req.getString("businessNum"));
            if(req.getString("fileNameXml")=="1"){
                reportFileEntity.setFileNameXml(req.getString("file_name"));
            } else if (req.getString("fileNameTxt")=="1") {
                reportFileEntity.setFileNameTxt(req.getString("file_name"));
            } else  {
                reportFileEntity.setFileNameModel(req.getString("file_name"));
            }
        }
        else {
            //1:xml文件
            if(req.getString("fileNameXml").equals("1")){
                String fileXml =  reportFileEntity.getFileNameXml();
                if(fileXml!=null){
                    File file = new File(xmlSourcePath +fileXml);
                    file.delete();
                }
                reportFileEntity.setFileNameXml(req.getString("file_name"));
            //2:txt json文件
            } else if (req.getString("fileNameTxt").equals("1")) {
                String fileTxt =  reportFileEntity.getFileNameTxt();
                if(fileTxt!=null){
                    File file = new File(jsonTargetPath +fileTxt);
                    file.delete();
                }
                reportFileEntity.setFileNameTxt(req.getString("file_name"));
            } else  {
                //3、模型文件
                String fileModel =  reportFileEntity.getFileNameModel();
                if(fileModel!=null){
                    File file = new File(modelPath +fileModel);
                    file.delete();
                }
                reportFileEntity.setFileNameModel(req.getString("file_name"));
            }
        }

        reportFileService.saveOrUpdate(reportFileEntity);
        return Result.success();
    }
}
